26.8. test — Regression tests package for Python — Python 3.6.12 documentation

# 26.8. `test` — Regression tests package for Python

Note

The `test` package is meant for internal use by Python only. It is documented for the benefit of the core developers of Python. Any use of this package outside of Python’s standard library is discouraged as code mentioned here can change or be removed without notice between releases of Python.

The `test` package contains all regression tests for Python as well as the modules `test.support` and `test.regrtest`. `test.support` is used to enhance your tests while `test.regrtest` drives the testing suite.

Each module in the `test` package whose name starts with `test_` is a testing suite for a specific module or feature. All new tests should be written using the `unittest` or `doctest` module. Some older tests are written using a “traditional” testing style that compares output printed to `sys.stdout`; this style of test is considered deprecated.

See also

Module `unittest`

  Writing PyUnit regression tests.

Module `doctest`

  Tests embedded in documentation strings.

## 26.8.1. Writing Unit Tests for the `test` package

It is preferred that tests that use the `unittest` module follow a few guidelines. One is to name the test module by starting it with `test_` and end it with the name of the module being tested. The test methods in the test module should start with `test_` and end with a description of what the method is testing. This is needed so that the methods are recognized by the test driver as test methods. Also, no documentation string for the method should be included. A comment (such as `# Tests function returns only True or False`) should be used to provide documentation for test methods. This is done because documentation strings get printed out if they exist and thus what test is being run is not stated.

A basic boilerplate is often used:

    import unittest
    from test import support

    class MyTestCase1(unittest.TestCase):

        # Only use setUp() and tearDown() if necessary

        def setUp(self):
            ... code to execute in preparation for tests ...

        def tearDown(self):
            ... code to execute to clean up after tests ...

        def test_feature_one(self):
            # Test feature one.
            ... testing code ...

        def test_feature_two(self):
            # Test feature two.
            ... testing code ...

        ... more test methods ...

    class MyTestCase2(unittest.TestCase):
        ... same structure as MyTestCase1 ...

    ... more test classes ...

    if __name__ == '__main__':
        unittest.main()

This code pattern allows the testing suite to be run by `test.regrtest`, on its own as a script that supports the `unittest` CLI, or via the `python -m unittest` CLI.

The goal for regression testing is to try to break code. This leads to a few guidelines to be followed:

- The testing suite should exercise all classes, functions, and constants. This includes not just the external API that is to be presented to the outside world but also “private” code.

- Whitebox testing (examining the code being tested when the tests are being written) is preferred. Blackbox testing (testing only the published user interface) is not complete enough to make sure all boundary and edge cases are tested.

- Make sure all possible values are tested including invalid ones. This makes sure that not only all valid values are acceptable but also that improper values are handled correctly.

- Exhaust as many code paths as possible. Test where branching occurs and thus tailor input to make sure as many different paths through the code are taken.

- Add an explicit test for any bugs discovered for the tested code. This will make sure that the error does not crop up again if the code is changed in the future.

- Make sure to clean up after your tests (such as close and remove all temporary files).

- If a test is dependent on a specific condition of the operating system then verify the condition already exists before attempting the test.

- Import as few modules as possible and do it as soon as possible. This minimizes external dependencies of tests and also minimizes possible anomalous behavior from side-effects of importing a module.

- Try to maximize code reuse. On occasion, tests will vary by something as small as what type of input is used. Minimize code duplication by subclassing a basic test class with a class that specifies the input:

      class TestFuncAcceptsSequencesMixin:

          func = mySuperWhammyFunction

          def test_func(self):
              self.func(self.arg)

      class AcceptLists(TestFuncAcceptsSequencesMixin, unittest.TestCase):
          arg = [1, 2, 3]

      class AcceptStrings(TestFuncAcceptsSequencesMixin, unittest.TestCase):
          arg = 'abc'

      class AcceptTuples(TestFuncAcceptsSequencesMixin, unittest.TestCase):
          arg = (1, 2, 3)

  When using this pattern, remember that all classes that inherit from `unittest.TestCase` are run as tests. The `Mixin` class in the example above does not have any data and so can’t be run by itself, thus it does not inherit from `unittest.TestCase`.

See also

Test Driven Development

  A book by Kent Beck on writing tests before code.

## 26.8.2. Running tests using the command-line interface

The `test` package can be run as a script to drive Python’s regression test suite, thanks to the `-m` option: python -m test. Under the hood, it uses `test.regrtest`; the call python -m test.regrtest used in previous Python versions still works. Running the script by itself automatically starts running all regression tests in the `test` package. It does this by finding all modules in the package whose name starts with `test_`, importing them, and executing the function `test_main()` if present or loading the tests via unittest.TestLoader.loadTestsFromModule if `test_main` does not exist. The names of tests to execute may also be passed to the script. Specifying a single regression test (python -m test test_spam) will minimize output and only print whether the test passed or failed.

Running `test` directly allows what resources are available for tests to use to be set. You do this by using the `-u` command-line option. Specifying `all` as the value for the `-u` option enables all possible resources: python -m test -uall. If all but one resource is desired (a more common case), a comma-separated list of resources that are not desired may be listed after `all`. The command python -m test -uall,-audio,-largefile will run `test` with all resources except the `audio` and `largefile` resources. For a list of all resources and more command-line options, run python -m test -h.

Some other ways to execute the regression tests depend on what platform the tests are being executed on. On Unix, you can run make test at the top-level directory where Python was built. On Windows, executing rt.bat from your `PCBuild` directory will run all regression tests.

# 26.9. `test.support` — Utilities for the Python test suite

The `test.support` module provides support for Python’s regression test suite.

Note

`test.support` is not a public module. It is documented here to help Python developers write tests. The API of this module is subject to change without backwards compatibility concerns between releases.

This module defines the following exceptions:

exception `test.support.` `TestFailed`

  Exception to be raised when a test fails. This is deprecated in favor of `unittest`-based tests and `unittest.TestCase`’s assertion methods.

exception `test.support.` `ResourceDenied`

  Subclass of `unittest.SkipTest`. Raised when a resource (such as a network connection) is not available. Raised by the `requires()` function.

The `test.support` module defines the following constants:

`test.support.` `verbose`

  `True` when verbose output is enabled. Should be checked when more detailed information is desired about a running test. verbose is set by `test.regrtest`.

`test.support.` `is_jython`

  `True` if the running interpreter is Jython.

`test.support.` `TESTFN`

  Set to a name that is safe to use as the name of a temporary file. Any temporary file that is created should be closed and unlinked (removed).

The `test.support` module defines the following functions:

`test.support.` `forget`(module_name)

  Remove the module named module_name from `sys.modules` and delete any byte-compiled files of the module.

`test.support.` `is_resource_enabled`(resource)

  Return `True` if resource is enabled and available. The list of available resources is only set when `test.regrtest` is executing the tests.

`test.support.` `requires`(resource, msg=None)

  Raise `ResourceDenied` if resource is not available. msg is the argument to `ResourceDenied` if it is raised. Always returns `True` if called by a function whose `__name__` is `'__main__'`. Used when tests are executed by `test.regrtest`.

`test.support.` `findfile`(filename, subdir=None)

  Return the path to the file named filename. If no match is found filename is returned. This does not equal a failure since it could be the path to the file.

  > Setting subdir indicates a relative path to use to find the file rather than looking directly in the path directories.

`test.support.` `run_unittest`(*classes)

  Execute `unittest.TestCase` subclasses passed to the function. The function scans the classes for methods starting with the prefix `test_` and executes the tests individually.

  It is also legal to pass strings as parameters; these should be keys in `sys.modules`. Each associated module will be scanned by `unittest.TestLoader.loadTestsFromModule()`. This is usually seen in the following `test_main()` function:

      def test_main():
          support.run_unittest(__name__)

  This will run all tests defined in the named module.

`test.support.` `run_doctest`(module, verbosity=None)

  Run `doctest.testmod()` on the given module. Return `(failure_count, test_count)`.

  If verbosity is `None`, `doctest.testmod()` is run with verbosity set to `verbose`. Otherwise, it is run with verbosity set to `None`.

`test.support.` `check_warnings`(*filters, quiet=True)

  A convenience wrapper for `warnings.catch_warnings()` that makes it easier to test that a warning was correctly raised. It is approximately equivalent to calling `warnings.catch_warnings(record=True)` with `warnings.simplefilter()` set to `always` and with the option to automatically validate the results that are recorded.

  `check_warnings` accepts 2-tuples of the form `("message regexp", WarningCategory)` as positional arguments. If one or more filters are provided, or if the optional keyword argument quiet is `False`, it checks to make sure the warnings are as expected: each specified filter must match at least one of the warnings raised by the enclosed code or the test fails, and if any warnings are raised that do not match any of the specified filters the test fails. To disable the first of these checks, set quiet to `True`.

  If no arguments are specified, it defaults to:

      check_warnings(("", Warning), quiet=True)

  In this case all warnings are caught and no errors are raised.

  On entry to the context manager, a `WarningRecorder` instance is returned. The underlying warnings list from `catch_warnings()` is available via the recorder object’s `warnings` attribute. As a convenience, the attributes of the object representing the most recent warning can also be accessed directly through the recorder object (see example below). If no warning has been raised, then any of the attributes that would otherwise be expected on an object representing a warning will return `None`.

  The recorder object also has a `reset()` method, which clears the warnings list.

  The context manager is designed to be used like this:

      with check_warnings(("assertion is always true", SyntaxWarning),
                          ("", UserWarning)):
          exec('assert(False, "Hey!")')
          warnings.warn(UserWarning("Hide me!"))

  In this case if either warning was not raised, or some other warning was raised, `check_warnings()` would raise an error.

  When a test needs to look more deeply into the warnings, rather than just checking whether or not they occurred, code like this can be used:

      with check_warnings(quiet=True) as w:
          warnings.warn("foo")
          assert str(w.args[0]) == "foo"
          warnings.warn("bar")
          assert str(w.args[0]) == "bar"
          assert str(w.warnings[0].args[0]) == "foo"
          assert str(w.warnings[1].args[0]) == "bar"
          w.reset()
          assert len(w.warnings) == 0

  Here all warnings will be caught, and the test code tests the captured warnings directly.

  Changed in version 3.2: New optional arguments filters and quiet.

`test.support.` `captured_stdin`()
`test.support.` `captured_stdout`()
`test.support.` `captured_stderr`()

  A context managers that temporarily replaces the named stream with `io.StringIO` object.

  Example use with output streams:

      with captured_stdout() as stdout, captured_stderr() as stderr:
          print("hello")
          print("error", file=sys.stderr)
      assert stdout.getvalue() == "hello\n"
      assert stderr.getvalue() == "error\n"

  Example use with input stream:

      with captured_stdin() as stdin:
          stdin.write('hello\n')
          stdin.seek(0)
          # call test code that consumes from sys.stdin
          captured = input()
      self.assertEqual(captured, "hello")

`test.support.` `temp_dir`(path=None, quiet=False)

  A context manager that creates a temporary directory at path and yields the directory.

  If path is `None`, the temporary directory is created using `tempfile.mkdtemp()`. If quiet is `False`, the context manager raises an exception on error. Otherwise, if path is specified and cannot be created, only a warning is issued.

`test.support.` `change_cwd`(path, quiet=False)

  A context manager that temporarily changes the current working directory to path and yields the directory.

  If quiet is `False`, the context manager raises an exception on error. Otherwise, it issues only a warning and keeps the current working directory the same.

`test.support.` `temp_cwd`(name='tempcwd', quiet=False)

  A context manager that temporarily creates a new directory and changes the current working directory (CWD).

  The context manager creates a temporary directory in the current directory with name name before temporarily changing the current working directory. If name is `None`, the temporary directory is created using `tempfile.mkdtemp()`.

  If quiet is `False` and it is not possible to create or change the CWD, an error is raised. Otherwise, only a warning is raised and the original CWD is used.

`test.support.` `temp_umask`(umask)

  A context manager that temporarily sets the process umask.

`test.support.` `can_symlink`()

  Return `True` if the OS supports symbolic links, `False` otherwise.

`@``test.support.` `skip_unless_symlink`

  A decorator for running tests that require support for symbolic links.

`@``test.support.` `anticipate_failure`(condition)

  A decorator to conditionally mark tests with `unittest.expectedFailure()`. Any use of this decorator should have an associated comment identifying the relevant tracker issue.

`@``test.support.` `run_with_locale`(catstr, *locales)

  A decorator for running a function in a different locale, correctly resetting it after it has finished. catstr is the locale category as a string (for example `"LC_ALL"`). The locales passed will be tried sequentially, and the first valid locale will be used.

`test.support.` `make_bad_fd`()

  Create an invalid file descriptor by opening and closing a temporary file, and returning its descriptor.

`test.support.` `import_module`(name, deprecated=False)

  This function imports and returns the named module. Unlike a normal import, this function raises `unittest.SkipTest` if the module cannot be imported.

  Module and package deprecation messages are suppressed during this import if deprecated is `True`.

  New in version 3.1.

`test.support.` `import_fresh_module`(name, fresh=(), blocked=(), deprecated=False)

  This function imports and returns a fresh copy of the named Python module by removing the named module from `sys.modules` before doing the import. Note that unlike `reload()`, the original module is not affected by this operation.

  fresh is an iterable of additional module names that are also removed from the `sys.modules` cache before doing the import.

  blocked is an iterable of module names that are replaced with `None` in the module cache during the import to ensure that attempts to import them raise `ImportError`.

  The named module and any modules named in the fresh and blocked parameters are saved before starting the import and then reinserted into `sys.modules` when the fresh import is complete.

  Module and package deprecation messages are suppressed during this import if deprecated is `True`.

  This function will raise `ImportError` if the named module cannot be imported.

  Example use:

      # Get copies of the warnings module for testing without affecting the
      # version being used by the rest of the test suite. One copy uses the
      # C implementation, the other is forced to use the pure Python fallback
      # implementation
      py_warnings = import_fresh_module('warnings', blocked=['_warnings'])
      c_warnings = import_fresh_module('warnings', fresh=['_warnings'])

  New in version 3.1.

`test.support.` `bind_port`(sock, host=HOST)

  Bind the socket to a free port and return the port number. Relies on ephemeral ports in order to ensure we are using an unbound port. This is important as many tests may be running simultaneously, especially in a buildbot environment. This method raises an exception if the `sock.family` is `AF_INET` and `sock.type` is `SOCK_STREAM`, and the socket has `SO_REUSEADDR` or `SO_REUSEPORT` set on it. Tests should never set these socket options for TCP/IP sockets. The only case for setting these options is testing multicasting via multiple UDP sockets.

  Additionally, if the `SO_EXCLUSIVEADDRUSE` socket option is available (i.e. on Windows), it will be set on the socket. This will prevent anyone else from binding to our host/port for the duration of the test.

`test.support.` `find_unused_port`(family=socket.AF_INET, socktype=socket.SOCK_STREAM)

  Returns an unused port that should be suitable for binding. This is achieved by creating a temporary socket with the same family and type as the `sock` parameter (default is `AF_INET`, `SOCK_STREAM`), and binding it to the specified host address (defaults to `0.0.0.0`) with the port set to 0, eliciting an unused ephemeral port from the OS. The temporary socket is then closed and deleted, and the ephemeral port is returned.

  Either this method or `bind_port()` should be used for any tests where a server socket needs to be bound to a particular port for the duration of the test. Which one to use depends on whether the calling code is creating a python socket, or if an unused port needs to be provided in a constructor or passed to an external program (i.e. the `-accept` argument to openssl’s s_server mode). Always prefer `bind_port()` over `find_unused_port()` where possible. Using a hard coded port is discouraged since it can make multiple instances of the test impossible to run simultaneously, which is a problem for buildbots.

`test.support.` `load_package_tests`(pkg_dir, loader, standard_tests, pattern)

  Generic implementation of the `unittest` `load_tests` protocol for use in test packages. pkg_dir is the root directory of the package; loader, standard_tests, and pattern are the arguments expected by `load_tests`. In simple cases, the test package’s `__init__.py` can be the following:

      import os
      from test.support import load_package_tests

      def load_tests(*args):
          return load_package_tests(os.path.dirname(__file__), *args)

`test.support.` `detect_api_mismatch`(ref_api, other_api, *, ignore=())

  Returns the set of attributes, functions or methods of ref_api not found on other_api, except for a defined list of items to be ignored in this check specified in ignore.

  By default this skips private attributes beginning with ‘_’ but includes all magic methods, i.e. those starting and ending in ‘__’.

  New in version 3.5.

`test.support.` `check__all__`(test_case, module, name_of_module=None, extra=(), blacklist=())

  Assert that the `__all__` variable of module contains all public names.

  The module’s public names (its API) are detected automatically based on whether they match the public name convention and were defined in module.

  The name_of_module argument can specify (as a string or tuple thereof) what module(s) an API could be defined in in order to be detected as a public API. One case for this is when module imports part of its public API from other modules, possibly a C backend (like `csv` and its `_csv`).

  The extra argument can be a set of names that wouldn’t otherwise be automatically detected as “public”, like objects without a proper `__module__` attribute. If provided, it will be added to the automatically detected ones.

  The blacklist argument can be a set of names that must not be treated as part of the public API even though their names indicate otherwise.

  Example use:

      import bar
      import foo
      import unittest
      from test import support

      class MiscTestCase(unittest.TestCase):
          def test__all__(self):
              support.check__all__(self, foo)

      class OtherTestCase(unittest.TestCase):
          def test__all__(self):
              extra = {'BAR_CONST', 'FOO_CONST'}
              blacklist = {'baz'}  # Undocumented name.
              # bar imports part of its API from _bar.
              support.check__all__(self, bar, ('bar', '_bar'),
                                   extra=extra, blacklist=blacklist)

  New in version 3.6.

The `test.support` module defines the following classes:

class `test.support.` `TransientResource`(exc, **kwargs)

  Instances are a context manager that raises `ResourceDenied` if the specified exception type is raised. Any keyword arguments are treated as attribute/value pairs to be compared against any exception raised within the `with` statement. Only if all pairs match properly against attributes on the exception is `ResourceDenied` raised.

class `test.support.` `EnvironmentVarGuard`

  Class used to temporarily set or unset environment variables. Instances can be used as a context manager and have a complete dictionary interface for querying/modifying the underlying `os.environ`. After exit from the context manager all changes to environment variables done through this instance will be rolled back.

  Changed in version 3.1: Added dictionary interface.

`EnvironmentVarGuard.` `set`(envvar, value)

  Temporarily set the environment variable `envvar` to the value of `value`.

`EnvironmentVarGuard.` `unset`(envvar)

  Temporarily unset the environment variable `envvar`.

class `test.support.` `SuppressCrashReport`

  A context manager used to try to prevent crash dialog popups on tests that are expected to crash a subprocess.

  On Windows, it disables Windows Error Reporting dialogs using SetErrorMode.

  On UNIX, `resource.setrlimit()` is used to set `resource.RLIMIT_CORE`’s soft limit to 0 to prevent coredump file creation.

  On both platforms, the old value is restored by `__exit__()`.

class `test.support.` `WarningsRecorder`

  Class used to record warnings for unit tests. See documentation of `check_warnings()` above for more details.

class `test.support.` `FakePath`(path)

  Simple path-like object. It implements the `__fspath__()` method which just returns the path argument. If path is an exception, it will be raised in `__fspath__()`.